import Computer
import Horde
import Network
import OS
import UI
from util import RAND

STATE_IN = 1
STATE_AT = 2
STATE_OUT = 3
STATE_DYING = 4
STATE_STRAY = 5

# Offsets from upper right of computer
OFFSET_X = 20
OFFSET_Y = 3

# speed at which OS drops
GRAVITY = 3

# speed of moving Bill
SLOW = 0
FAST = 1

WCELS = 4                 # # of bill walking animation frames
DCELS = 5                 # # of bill dying animation frames
ACELS = 13                # # of bill switching OS frames

def load_pix():
	Bill.lcels = range(WCELS)
	Bill.rcels = range(WCELS)
	for i in range(WCELS - 1):
		Bill.lcels[i] = UI.load_picture_indexed("billL", i)
		Bill.rcels[i] = UI.load_picture_indexed("billR", i)

	Bill.lcels[WCELS - 1] = Bill.lcels[1]
	Bill.rcels[WCELS - 1] = Bill.rcels[1]

	Bill.dcels = range(DCELS)
	for i in range(DCELS):
		Bill.dcels[i] = UI.load_picture_indexed("billD", i)
	Bill.width = Bill.dcels[0].get_width()
	Bill.height = Bill.dcels[0].get_height()
	
	Bill.acels = range(ACELS)
	for i in range(ACELS):
		Bill.acels[i] = UI.load_picture_indexed("billA", i)

class Bill:
	# Adds a bill to the in state
	def __init__(self, parent, game):
		self.parent = parent
		self.state = STATE_IN
		self.x, self.y = self.get_border(game.screensize)
		self.index = 0
		self.cels = self.lcels
		self.cargo = OS.WINGDOWS
		self.x_offset = -2
		self.y_offset = -15
		self.target_c = game.network.computers[RAND(0, len(game.network.computers))]
		self.target_x = self.target_c.x + self.target_c.width - OFFSET_X
		self.target_y = self.target_c.y + OFFSET_Y
		parent.counters[Horde.COUNTER_ON] += 1
		parent.counters[Horde.COUNTER_OFF] += -1

	def get_border(self, screensize):
		i = RAND(0, 4)

		if i % 2 == 0:
			x = RAND(0, screensize - self.width)
		else:
			y = RAND(0, screensize - self.height)

		if i == 0:
			y = -self.height - 16
		elif i == 1:
			x = screensize + 1
		elif i == 2:
			y = screensize + 1
		elif i == 3:
			x = -self.width - 2
		return x, y

	#  Moves bill toward his target - returns whether or not he moved
	def move(self, mode, level):
		xdist = self.target_x - self.x
		ydist = self.target_y - self.y
		step = min(11 + level, 15)
		signx = xdist >= 0 and 1 or -1
		signy = ydist >= 0 and 1 or -1
		xdist = abs(xdist)
		ydist = abs(ydist)
		if not xdist and not ydist:
			return 0
		elif xdist < step and ydist < step:
			self.x = self.target_x
			self.y = self.target_y
		else:
			dx = (xdist * step * signx) / (xdist + ydist)
			dy = (ydist * step * signy) / (xdist + ydist)
			if mode == FAST:
				dx *= 1.25
				dy *= 1.25
			self.x += dx
			self.y += dy
			if dx < 0:
				self.cels = self.lcels
			elif dx > 0:
				self.cels = self.rcels
		return 1

	def draw_std(self, screen):
		if self.cargo >= 0:
			OS.draw(self.cargo, self.x + self.x_offset, self.y + self.y_offset, screen)
		screen.blit(self.cels[self.index], (self.x, self.y))

	def draw_at(self, screen):
		if self.index > 6 and self.index < 12:
			OS.draw(0, self.x + self.sx, self.y + self.sy, screen)
		if self.cargo >= 0:
			OS.draw(self.cargo, self.x + self.x_offset, self.y + self.y_offset, screen)
		screen.blit(self.cels[self.index], (self.target_c.x, self.target_c.y))

	def draw_stray(self, screen):
		OS.draw(self.cargo, self.x, self.y, screen)

	def draw(self, screen):
		if self.state in (STATE_IN, STATE_OUT, STATE_DYING):
			self.draw_std(screen)
		elif self.state == STATE_AT:
			self.draw_at(screen)
		elif self.state == STATE_STRAY:
			self.draw_stray(screen)

	# Update Bill's position
	def update_in(self, game):
		moved = self.move(SLOW, game.level)
		if not moved and self.target_c.os != OS.WINGDOWS and not self.target_c.busy:
			self.target_c.busy = 1
			self.cels = self.acels
			self.index = 0
			self.state = STATE_AT
			return
		elif not moved:
			while 1:
				i = game.network.computers[RAND(0, len(game.network.computers))]
				if i != self.target_c:
					break
			self.target_c = i
			self.target_x = i.x + i.width - OFFSET_X
			self.target_y = i.y + OFFSET_Y
		self.index += 1
		self.index %= WCELS
		self.y_offset += (8 * (self.index % 2) - 4)

	# Update Bill standing at a computer
	def update_at(self, game):
		if self.index == 0 and self.target_c.os == OS.OFF:
			self.index = 6
			if self.target_c.stray == None:
				self.cargo = -1
			else:
				self.cargo = self.target_c.stray.cargo
				self.parent.remove_bill(self.target_c.stray, game.network)
				self.target_c.stray = None
		else:
			self.index += 1
		if self.index == 13:
			self.y_offset = -15
			self.x_offset = -2
			self.target_x, self.target_y = self.get_border(game.screensize)
			self.index = 0
			self.cels = self.lcels
			self.state = STATE_OUT
			self.target_c.busy = 0
			return
		self.y_offset = self.height - OS.OS.height
		if self.index in (1, 2):
			self.x -= 8
			self.x_offset +=8
		elif self.index == 3:
			self.x -= 10
			self.x_offset +=10
		elif self.index == 4:
			self.x += 3
			self.x_offset -=3
		elif self.index == 5:
			self.x += 2
			self.x_offset -=2
		elif self.index == 6:
			if self.target_c.os != OS.OFF:
				game.network.counters[Network.COUNTER_BASE] += -1
				game.network.counters[Network.COUNTER_OFF] += 1
				self.cargo = self.target_c.os
			else:
				self.x -= 21
				self.x_offset += 21
			self.target_c.os = OS.OFF
			self.y_offset = -15
			self.x += 20
			self.x_offset -=20
		elif self.index == 7:
			self.sy = self.y_offset
			self.sx = -2
		elif self.index == 8:
			self.sy = -15
			self.sx = -2
		elif self.index == 9:
			self.sy = -7
			self.sx = -7
			self.x -= 8
			self.x_offset +=8
		elif self.index == 10:
			self.sy = 0
			self.sx = -7
			self.x -= 15
			self.x_offset +=15
		elif self.index == 11:
			self.sy = 0
			self.sx = -7
			self.target_c.os = OS.WINGDOWS
			game.network.counters[Network.COUNTER_OFF] += -1
			game.network.counters[Network.COUNTER_WIN] += 1
		elif self.index == 12:
			self.x += 11
			self.x_offset -= 11

	# Updates Bill fleeing with his ill gotten gain
	def update_out(self, game):
		if UI.intersect(self.x, self.y, self.width, self.height, 0, 0, game.screensize, game.screensize):
			self.move(FAST, game.level)
			self.index += 1
			self.index %= WCELS
			self.y_offset += (8 * (self.index % 2) - 4)
		else:
			self.parent.remove_bill(self, game.network)
			self.parent.counters[Horde.COUNTER_ON] += -1
			self.parent.counters[Horde.COUNTER_OFF] += 1

	# Updates a Bill who is dying
	def update_dying(self, game):
		if self.index < DCELS - 1:
			self.y_offset += (self.index * GRAVITY)
			self.index += 1
		else:
			self.y += self.y_offset
			if self.cargo < 0 or self.cargo == OS.WINGDOWS:
				self.parent.remove_bill(self, game.network)
			else:
				self.parent.move_bill(self)
				self.state = STATE_STRAY
			self.parent.counters[Horde.COUNTER_ON] += -1

	def update(self, game):
		if self.state == STATE_IN:
			self.update_in(game)
		elif self.state == STATE_AT:
			self.update_at(game)
		elif self.state == STATE_OUT:
			self.update_out(game)
		elif self.state == STATE_DYING:
			self.update_dying(game)

	def set_dying(self):
		self.index = -1
		self.cels = self.dcels
		self.x_offset = -2
		self.y_offset = -15
		self.state = STATE_DYING

	def clicked(self, locx, locy):
		return locx > self.x and locx < self.x + self.width and locy > self.y and locy < self.y + self.height

	def clickedstray(self, locx, locy):
		return locx > self.x and locx < self.x + OS.OS.width and locy > self.y and locy < self.y + OS.OS.height
