import pygame

import Network
import Spark
import OS
from util import RAND

class Cable:
	def __init__(self, parent):	# setup
		self.parent = parent
		self.c1 = parent.computers[RAND(0, len(parent.computers))]
		while 1:
			self.c2 = parent.computers[RAND(0, len(parent.computers))]
			if self.c2 != self.c1:
				break
		self.spark = None

		cwidth = self.c1.width
		cheight = self.c1.height
		self.x1 = self.c1.x + cwidth / 3
		self.x2 = self.c2.x + cwidth / 3
		self.y1 = self.c1.y + cheight / 2
		self.y2 = self.c2.y + cheight / 2

	def reverse(self):
		def SWAP(x, y):
			x, y = y, x
		SWAP(self.c1, self.c2)
		SWAP(self.x1, self.x2)
		SWAP(self.y1, self.y2)

	def draw(self, screen):
		pygame.draw.line(screen, (0, 0, 0), (self.x1, self.y1), (self.x2, self.y2), 2)
		if self.spark:
			self.spark.draw(screen)

	def update(self, computers):
		if self.spark:
			if (self.c1.os == OS.WINGDOWS) == (self.c2.os == OS.WINGDOWS):
				self.spark = None

			elif (self.c1.os == OS.WINGDOWS) or (self.c2.os == OS.WINGDOWS):
				if self.c2.os == OS.WINGDOWS:
					self.reverse()
	
				xdist = self.x2 - self.spark.x
				ydist = self.y2 - self.spark.y
	
				sx = xdist >= 0 and 1.0 or -1.0
				sy = ydist >= 0 and 1.0 or -1.0
				xdist = abs(xdist)
				ydist = abs(ydist)

				if xdist == 0 and ydist == 0:
					if not self.c2.busy:
						if self.c2.os == OS.OFF:
							counter = Network.COUNTER_OFF
						else:
							counter = Network.COUNTER_BASE
						self.parent.counters[counter] += -1
						self.parent.counters[Network.COUNTER_WIN] += 1
						self.c2.os = OS.WINGDOWS

					self.spark = None

				elif max(xdist, ydist) < Spark.SPEED:
					self.spark.x = self.x2
					self.spark.y = self.y2

				else:
					self.spark.fx += float(xdist * Spark.SPEED * sx) / float(xdist + ydist)
					self.spark.fy += float(ydist * Spark.SPEED * sy) / float(xdist + ydist)
					self.spark.x = int(self.spark.fx)
					self.spark.y = int(self.spark.fy)

				if self.spark:
					self.spark.index = 1 - self.spark.index

		else:
			if (self.c1.os == OS.WINGDOWS) == (self.c2.os == OS.WINGDOWS):
				pass
			elif (self.c1.os == OS.WINGDOWS) or (self.c2.os == OS.WINGDOWS):
				self.spark = Spark.Spark()
				if self.c2.os == OS.WINGDOWS:
					self.reverse()
				self.spark.x = self.x1
				self.spark.fx = self.x1
				self.spark.y = self.y1
				self.spark.fy = self.y1

	def onspark(self, locx, locy):
		if not self.spark:
			return 0
		return abs(locx - self.spark.x) < self.spark.width and abs(locy - self.spark.y) < self.spark.height

	def reset(self):
		self.spark = None
