import Bill
import Network
import OS
import UI
from util import RAND

TOASTER = 0			# computer 0 is a toaster
OS_OFFSET = 4			# offset of screen from 0,0
MIN_PC = 6			# type >= MIN_PC means the computer is a PC

cpuname = ("toaster", "maccpu", "nextcpu", "sgicpu",
				"suncpu", "palmcpu", "os2cpu", "bsdcpu")
NUM_SYS = len(cpuname)

def load_pix():
	Computer.pictures = range(NUM_SYS)
	for i in range(NUM_SYS):
		Computer.pictures[i] = UI.load_picture(cpuname[i])
	Computer.width = Computer.pictures[0].get_width()
	Computer.height = Computer.pictures[0].get_height()

class Computer:
	def __init__(self, parent, index, screensize):	# setup
		self.parent = parent
		counter = 0
		flag = 0
		border = screensize / 10
		while not flag:
			counter += 1
			if counter > 4000:
				return None
			self.x = RAND(border, screensize - border - self.width)
			self.y = RAND(border, screensize - border - self.height)
			flag = 1
			# check for conflicting computer placement
			for j in range(index):
				c = parent.computers[j]
				twidth = self.width - Bill.OFFSET_X + Bill.Bill.width
				if UI.intersect(self.x, self.y, twidth, self.height, c.x, c.y, twidth, c.height):
					flag = 0
					break

		self.type = RAND(1, NUM_SYS)
		self.os = self.determineOS()
		self.busy = 0
		self.stray = None

	def determineOS(self):
		if self.type < MIN_PC:
			return self.type
		else:
			return OS.randpc()

	def on(self, locx, locy):
		return abs(locx - self.x) < self.width and abs(locy - self.y) < self.height

	def compatible(self, system):
		return self.type == system or (self.type >= MIN_PC and OS.ispc(system))

	def draw(self, screen):
		screen.blit(self.pictures[self.type], (self.x, self.y))
		if self.os != OS.OFF:
			OS.draw(self.os, self.x + OS_OFFSET, self.y + OS_OFFSET, screen)
