import Cable
import Computer
import OS

COUNTER_OFF = 0
COUNTER_BASE = 1
COUNTER_WIN = 2
COUNTER_MAX = 3

STD_MAX_COMPUTERS = 20

class Network:
	def __init__(self):
		self.counters = range(COUNTER_MAX)

	# sets up network for each level
	def setup(self, game):
		self.computers = []
		ncomputers = int(min(8 + game.level, STD_MAX_COMPUTERS) * game.scale(2))
		for i in range(ncomputers):
			computer = Computer.Computer(self, i, game.screensize)
			if not computer:
				break
			self.computers.append(computer)
		ncomputers = len(self.computers)
		self.counters[COUNTER_OFF] = 0
		self.counters[COUNTER_BASE] = ncomputers
		self.counters[COUNTER_WIN] = 0

		self.cables = []
		ncables = min(game.level, ncomputers / 2)
		for None in range(ncables):
			self.cables.append(Cable.Cable(self))

	def draw(self, screen):
		for cable in self.cables:
			cable.draw(screen)
		for computer in self.computers:
			computer.draw(screen)

	def update(self):
		for cable in self.cables:
			cable.update(self.computers)

	def toasters(self):
		for computer in self.computers:
			computer.type = Computer.TOASTER
			computer.os = OS.OFF
		self.cables = []

	def clear_stray(self, bill):
		for computer in self.computers:
			if computer.stray == bill:
				computer.stray = None
